#!/usr/bin/env python3

# -- nercone-modern --------------------------------------------- #
# logging.py on nercone-modern                                    #
# Made by DiamondGotCat, Licensed under MIT License               #
# Copyright (c) 2025 DiamondGotCat                                #
# ---------------------------------------------- DiamondGotCat -- #

import sys
from .color import ModernColor
from datetime import datetime, timezone

ModernLoggingLevels = ["DEBUG", "INFO", "WARN", "ERROR", "CRITICAL"]
MAX_LOG_LEVEL_WIDTH = max(len(level) for level in ModernLoggingLevels)
LEVEL_ALIASES = {
    "D": "DEBUG",
    "DEBUG": "DEBUG",
    "I": "INFO",
    "INFO": "INFO",
    "INFORMATION": "INFO",
    "W": "WARN",
    "WARN": "WARN",
    "WARNING": "WARN",
    "E": "ERROR",
    "ERROR": "ERROR",
    "C": "CRITICAL",
    "CRITICAL": "CRITICAL"
}

_last_process = None
_last_level = None
_max_proc_width = 0
_max_prefix_width = 20

def normalize_level(level: str) -> str:
    level = level.strip().upper()
    return LEVEL_ALIASES.get(level, level)

def is_higher_priority(level_a: str, level_b: str) -> bool:
    a = normalize_level(level_a)
    b = normalize_level(level_b)
    try:
        return ModernLoggingLevels.index(a) >= ModernLoggingLevels.index(b)
    except ValueError:
        raise ValueError(f"Unknown log level: {level_a} or {level_b}")

class ModernLogging:
    def __init__(self, process_name: str = "App", display_level: str = "INFO", filepath: str | None = None):
        self.process_name = process_name
        self.display_level = display_level
        self.filepath = filepath
        self.short_mode = short_mode
        global _max_proc_width
        _max_proc_width = max(_max_proc_width, len(process_name))

    def log(self, message: str = "", level_text: str = "INFO", level_color: str | None = None):
        if not is_higher_priority(level_text, self.display_level):
            return
        global _last_process, _last_level
        log_line = self.make(message=message, level_text=level_text, level_color=level_color)
        print(log_line)
        _last_process = self.process_name
        _last_level = normalize_level(level_text.strip().upper())
        if self.filepath:
            with open(self.filepath, "a") as f:
                f.write(f"{log_line}\n")

    def prompt(self, message: str = "", level_text: str = "INFO", level_color: str | None = None, default: str | None = None, show_default: bool = False, choices: list[str] | None = None, show_choices: bool = True, interrupt_ignore: bool = False, interrupt_default: str | None = None) -> str:
        if not is_higher_priority(level_text, self.display_level):
            return
        global _last_process, _last_level
        if default and show_default:
            message += f" ({default})"
        if choices and show_choices:
            message += f" [{'/'.join(choices)}]"
        if not message.endswith(" "):
            message += " "
        log_line = self.make(message=message, level_text=level_text, level_color=level_color)
        print(log_line, end="")
        _last_process = self.process_name
        _last_level = normalize_level(level_text.strip().upper())
        answer = ""
        used_default = False
        try:
            answer = input()
        except KeyboardInterrupt:
            if interrupt_ignore:
                if interrupt_default:
                    answer = interrupt_default
                    used_default = True
                print()
            else:
                raise
        if answer.strip() == "" and default is not None:
            if choices:
                selected_default = self._select_choice(default, choices)
                if selected_default is not None:
                    answer = default
                    used_default = True
            else:
                answer = default
                used_default = True
        if used_default:
            self._rewrite_prompt_line_with_answer(log_line, answer)
        if self.filepath:
            with open(self.filepath, "a") as f:
                f.write(f"{log_line}{answer}\n")
        if choices:
            selected = self._select_choice(answer, choices)
            if selected is not None:
                return selected
            else:
                while True:
                    log_line = self.make(message=f"Invalid selection. Please select from: {'/'.join(choices)}", level_text=level_text, level_color=level_color)
                    print(log_line)
                    if self.filepath:
                        with open(self.filepath, "a") as f:
                            f.write(f"{log_line}{answer}\n")
                    log_line = self.make(message=message, level_text=level_text, level_color=level_color)
                    print(log_line, end="")
                    try:
                        answer = input()
                    except KeyboardInterrupt:
                        if interrupt_ignore:
                            if interrupt_default:
                                answer = interrupt_default
                                used_default = True
                            print()
                        else:
                            raise
                    used_default = False
                    if answer.strip() == "" and default is not None:
                        if choices:
                            selected_default = self._select_choice(default, choices)
                            if selected_default is not None:
                                answer = default
                                used_default = True
                        else:
                            answer = default
                            used_default = True
                    if used_default:
                        self._rewrite_prompt_line_with_answer(log_line, answer)
                    if self.filepath:
                        with open(self.filepath, "a") as f:
                            f.write(f"{log_line}{answer}\n")
                    if answer.strip() == "" and default is not None:
                        if choices:
                            selected_default = self._select_choice(default, choices)
                            if selected_default is not None:
                                return default
                        else:
                            return default
                    selected = self._select_choice(answer, choices)
                    if selected is not None:
                        return selected
        return answer

    def _rewrite_prompt_line_with_answer(self, log_line: str, answer: str) -> None:
        try:
            sys.stdout.write("\033[F\r")
            sys.stdout.write(f"{log_line}{answer}  \n")
            sys.stdout.flush()
        except Exception:
            print(f"{log_line}{answer}")

    def _select_choice(self, answer: str, choices: list[str]) -> str | None:
        if answer in choices:
            return answer
        stripped = answer.strip()
        if stripped in choices:
            return stripped
        lower_map = {c.lower(): c for c in choices}
        if answer.lower() in lower_map:
            return lower_map[answer.lower()]
        if stripped.lower() in lower_map:
            return lower_map[stripped.lower()]
        return None

    def make(self, message: str = "", level_text: str = "INFO", level_color: str | None = None):
        level_text = normalize_level(level_text.strip().upper())
        if not level_color:
            if level_text == "DEBUG":
                level_color = 'gray'
            elif level_text == "INFO":
                level_color = 'blue'
            elif level_text == "WARN":
                level_color = 'yellow'
            elif level_text == "ERROR":
                level_color = 'red'
            elif level_text == "CRITICAL":
                level_color = 'red'
            else:
                level_color = 'blue'
        return self._make(message=message, level_text=level_text, level_color=level_color)

    def _current_time(self) -> str:
        return datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%SZ')

    def _make(self, message: str = "", level_text: str = "INFO", level_color: str = "blue"):
        global _max_proc_width, _max_prefix_width
        timestamp = self._current_time()
        log_level = level_text.ljust(MAX_LOG_LEVEL_WIDTH)
        proc_name = self.process_name.ljust(_max_proc_width)
        _max_prefix_width = max(_max_prefix_width, len(f"[{timestamp} {log_level} {proc_name}]"))
        return f"[{timestamp} {ModernColor.color(level_color)}{log_level}{ModernColor.color('reset')} {proc_name}] {message}"



#!/usr/bin/env python3

# -- nercone-modern --------------------------------------------- #
# __main__.py on nercone-modern                                   #
# Made by DiamondGotCat, Licensed under MIT License               #
# Copyright (c) 2025 DiamondGotCat                                #
# ---------------------------------------------- DiamondGotCat -- #

import time
from nercone_modern.logging import ModernLogging
from nercone_modern.progressbar import ModernProgressBar

logger0 = ModernLogging("Demo", display_level="DEBUG")
logger1 = ModernLogging("Main", display_level="DEBUG")
logger2 = ModernLogging("Sub", display_level="DEBUG")

try:
    logger1.log("This is a debug message", "DEBUG")
    logger1.log("This is a info message", "INFO")
    logger1.log("This is a info message", "INFO")
    logger1.log("This is a info message", "INFO")
    logger2.log("This is a info message", "INFO")
    logger1.log("This is a warning message", "WARNING")
    logger1.log("This is a error message", "ERROR")
    logger1.log("This is a critical error message", "CRITICAL")
    prompt_result = logger1.prompt("Continue demo?", default="Y", choices=["Y", "n"], interrupt_ignore=True, interrupt_default="n")
    if prompt_result == "n":
        logger1.log("Exiting demo. See you!", "INFO")
        raise SystemExit(0)

    progress_bar1 = ModernProgressBar(total=100, process_name="Task 1", spinner_mode=False)
    progress_bar1.setMessage("Waiting...")
    progress_bar2 = ModernProgressBar(total=1, process_name="Task 2", spinner_mode=True)
    progress_bar2.setMessage("Waiting...")

    progress_bar1.start()
    progress_bar2.start()

    progress_bar1.setMessage("Running...")
    for i in range(100):
        time.sleep(0.05)
        progress_bar1.update()
    progress_bar1.setMessage("Done!")
    progress_bar1.finish()

    progress_bar2.spin_start()
    progress_bar2.setMessage("Running with Spinner Mode...")
    time.sleep(10)
    progress_bar2.setMessage("Done!")
    progress_bar2.finish()
except KeyboardInterrupt:
    print()
    logger0.log("Aborted.", "INFO")



#!/usr/bin/env python3

# -- nercone-modern --------------------------------------------- #
# progressbar.py on nercone-modern                                #
# Made by DiamondGotCat, Licensed under MIT License               #
# Copyright (c) 2025 DiamondGotCat                                #
# ---------------------------------------------- DiamondGotCat -- #

import sys
import threading
from . import logging
from .color import ModernColor
from .logging import ModernLogging

class ModernProgressBar:
    _active_bars = []
    _last_rendered = False
    _lock = threading.RLock()

    def __init__(self, total: int, process_name: str, spinner_mode: bool = False, primary_bar: str = "-", secondary_bar: str = "-", centor_bar: str = "-", primary_color: str = "blue", secondary_color: str = "green", third_color: str = "white", box_color: str = "reset", box_left: str = "[", box_right: str = "]"):
        self.total = total
        self.process_name = process_name.strip()
        self.spinner_mode = spinner_mode
        self.primary_bar = primary_bar
        self.secondary_bar = secondary_bar
        self.centor_bar = centor_bar
        self.primary_color = primary_color
        self.secondary_color = secondary_color
        self.third_color = third_color
        self.box_color = box_color
        self.box_left = box_left
        self.box_right = box_right
        self.current = 0
        self.index = len(ModernProgressBar._active_bars)
        ModernProgressBar._active_bars.append(self)
        self.log_lines = 0
        self.step = 0
        self.spinner_step = 0
        self.message = "No message"
        self._spinner_thread = None
        self._spinner_stop_event = threading.Event()
        self._spinner_ready = False
        self._initial_render()

    def _initial_render(self):
        print()

    def spinner(self, enabled: bool = True):
        if self.spinner_mode == enabled:
            return
        self.spinner_mode = enabled
        if not self._spinner_ready:
            self._render(advance_spinner=False)
            return
        if enabled:
            self._start_spinner_thread_if_needed()
        else:
            self._stop_spinner_thread()
            self._render(advance_spinner=False)

    def spin_start(self):
        if self._spinner_ready and self.spinner_mode:
            return
        self._spinner_ready = True
        self.spinner_mode = True
        self.spinner_step = 0
        self._start_spinner_thread_if_needed()
        self._render(advance_spinner=False)

    def setMessage(self, message: str = ""):
        self.message = message

    def start(self):
        self._render(advance_spinner=False)
        self._start_spinner_thread_if_needed()

    def update(self, amount: int = 1):
        if self._should_spin():
            self._render(advance_spinner=False)
            return
        self.current += amount
        if self.current > self.total:
            self.current = self.total
        self._render(advance_spinner=False)

    def finish(self):
        self.current = self.total
        self.spinner_mode = False
        self._spinner_ready = False
        self._stop_spinner_thread()
        self._render(final=True, advance_spinner=False)

    def log(self, message: str = "", level_text: str = "INFO", level_color: str | None = None, show_proc: bool | None = None, show_level: bool | None = None, modernLogging: ModernLogging = None):
        with ModernProgressBar._lock:
            self.log_lines = 0
            if modernLogging is None:
                modernLogging = ModernLogging(self.process_name)
            result = modernLogging.make(message=message, level_text=level_text, level_color=level_color, show_proc=show_proc, show_level=show_level)
            if self.log_lines > 0:
                move_up = self.log_lines
            else:
                move_up = len(ModernProgressBar._active_bars) - self.index
            sys.stdout.write(f"\033[{move_up}A")
            sys.stdout.write("\033[K")
            print(result)
            self.log_lines += 1
            self._render(advance_spinner=False)

    def _start_spinner_thread_if_needed(self):
        if not self._should_spin():
            return
        if self._spinner_thread and self._spinner_thread.is_alive():
            return
        self._spinner_stop_event = threading.Event()
        self._spinner_thread = threading.Thread(target=self._spinner_worker, daemon=True)
        self._spinner_thread.start()

    def _stop_spinner_thread(self):
        if self._spinner_thread and self._spinner_thread.is_alive():
            self._spinner_stop_event.set()
            self._spinner_thread.join()
        self._spinner_thread = None

    def _spinner_worker(self):
        while not self._spinner_stop_event.wait(0.05):
            if not self._should_spin():
                continue
            self._render()

    def _render(self, final: bool = False, advance_spinner: bool = True):
        with ModernProgressBar._lock:
            progress = self.current / self.total if self.total else 0
            bar = self._progress_bar(progress, advance_spinner=advance_spinner and self._should_spin())
            percentage_value = int(round(min(max(progress, 0), 1) * 100))
            percentage = f"{percentage_value:3d}%"
            if self.current == self.total:
                percentage = "DONE"
            percentage_alt = "    "
            if self.spinner_mode:
                if self._should_spin():
                    percentage_alt = "RUNN"
                else:
                    percentage_alt = "WAIT"
            name_width = max(len(bar.process_name) for bar in ModernProgressBar._active_bars) if ModernProgressBar._active_bars else len(self.process_name)
            proc_name = f"{self.process_name:<{name_width}}"
            total_width = max(len(str(bar.total)) for bar in ModernProgressBar._active_bars) if ModernProgressBar._active_bars else max(len(str(self.total)), 1)
            status = ""
            if not (final or (self.spinner_mode and self._spinner_ready)):
                if self.spinner_mode:
                    status = f" ({' ' * total_width}/{' ' * total_width})"
                else:
                    status = f" ({self.current:>{total_width}}/{self.total:>{total_width}})"
            line = f"{ModernColor.color(self.box_color)}{self.box_left}{ModernColor.color('reset')}{ModernColor.color('gray')}{bar}{ModernColor.color('reset')}{ModernColor.color(self.box_color)}{self.box_right}{ModernColor.color('reset')} {ModernColor.color(self.primary_color)}{proc_name}{ModernColor.color('reset')} {ModernColor.color(self.secondary_color)}{percentage_alt if self.spinner_mode else percentage}{ModernColor.color('reset')}{ModernColor.color('gray')}{status}{ModernColor.color('reset')} {self.message}"
            total_move_up = self.log_lines + (len(ModernProgressBar._active_bars) - self.index)
            if total_move_up > 0:
                sys.stdout.write(f"\033[{total_move_up}A")
            sys.stdout.write("\r")
            sys.stdout.write("\033[K")
            sys.stdout.write(line)
            sys.stdout.write("\n")
            down_lines = max(total_move_up - 1, 0)
            if down_lines > 0:
                sys.stdout.write(f"\033[{down_lines}B")
            sys.stdout.flush()

    def _progress_bar(self, progress: int, advance_spinner: bool = True):
        bar_length = logging._max_prefix_width - 3
        if not self._should_spin():
            if self.current == self.total:
                center_bar = ""
            else:
                center_bar = self.centor_bar
            if self.current <= 0 and not self._spinner_ready:
                return f"{ModernColor.color('gray')}{self.secondary_bar * (bar_length + 1)}"
            filled_length = int(progress * bar_length) + 1
            return f"{ModernColor.color(self.primary_color)}{self.primary_bar * filled_length}{ModernColor.color(self.third_color)}{center_bar}{ModernColor.color('gray')}{self.secondary_bar * (bar_length - filled_length)}"
        else:
            if self.current <= 0 and not self._spinner_ready:
                return f"{ModernColor.color('gray')}{'-' * (bar_length + 1)}"
            spinner_symbol_length = 1
            spinner_end_bar_length = bar_length - self.spinner_step
            spinner_start_bar_length = bar_length - spinner_end_bar_length
            if advance_spinner:
                self.spinner_step = (self.spinner_step + 1) % (bar_length + 1)
            return f"{ModernColor.color('gray')}{'-' * spinner_start_bar_length}{ModernColor.color(self.third_color)}{'-' * spinner_symbol_length}{ModernColor.color('gray')}{'-' * spinner_end_bar_length}"

    def _should_spin(self):
        return self.spinner_mode and self._spinner_ready



#!/usr/bin/env python3

# -- nercone-modern --------------------------------------------- #
# text.py on nercone-modern                                       #
# Made by DiamondGotCat, Licensed under MIT License               #
# Copyright (c) 2025 DiamondGotCat                                #
# ---------------------------------------------- DiamondGotCat -- #

from typing import Union
from .color import ModernColor

class ModernText:
    def __init__(self, content="", color: str = ModernColor.WHITE):
        self.content = content
        if not color.startswith("\033"):
            color = getattr(ModernColor, color.upper(), ModernColor.WHITE)
        self.color = color

    def __add__(self, other: Union[str, "ModernText"]):
        if isinstance(other, ModernText):
            if self.color == other.color:
                return ModernText(self.content + other.content, self.color)
            else:
                combined = f"{self.color}{self.content}{ModernColor.RESET}{other.color}{other.content}"
                return ModernText(combined, ModernColor.RESET)
        elif isinstance(other, str):
            return ModernText(self.content + other, self.color)
        else:
            raise TypeError(f"Unsupported operand type(s) for +: 'ModernText' and '{type(other).__name__}'")

    def __str__(self):
        return f"{self.color}{self.content}{ModernColor.RESET}"



#!/usr/bin/env python3

# -- nercone-modern --------------------------------------------- #
# color.py on nercone-modern                                      #
# Made by DiamondGotCat, Licensed under MIT License               #
# Copyright (c) 2025 DiamondGotCat                                #
# ---------------------------------------------- DiamondGotCat -- #

class ModernColor:
    def color_by_code(color_code: int | str = 0):
        return f"\033[{color_code}m"

    def color(color_name: str = "reset"):
        if color_name in ("reset", "default"):
            return ModernColor.color_by_code(0)
        elif color_name == "black":
            return ModernColor.color_by_code(30)
        elif color_name == "red":
            return ModernColor.color_by_code(31)
        elif color_name == "green":
            return ModernColor.color_by_code(32)
        elif color_name == "yellow":
            return ModernColor.color_by_code(33)
        elif color_name == "blue":
            return ModernColor.color_by_code(34)
        elif color_name == "magenta":
            return ModernColor.color_by_code(35)
        elif color_name == "cyan":
            return ModernColor.color_by_code(36)
        elif color_name == "white":
            return ModernColor.color_by_code(37)
        elif color_name in ("gray", "grey"):
            return ModernColor.color_by_code(90)
        else:
            return ""

    RESET = "\033[0m"
    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    GRAY = "\033[90m"
